﻿using System;
using System.Windows.Forms;
using BrightIdeasSoftware;
using DoenaSoft.DVDProfiler.CastDemoClient_V2.Resources;
using System.Collections.Generic;

namespace DoenaSoft.DVDProfiler.CastDemoClient_V2
{
    internal partial class CastListControl : UserControl
    {
        public const String NewId = "<newid> ";

        internal Boolean ShowAddEpisodeButton;

        internal String Upc;

        internal event EventHandler<AddedEventArgs> CastEntryAdded;

        public CastListControl()
        {
            InitializeComponent();
        }

        private void CastListControl_Load(object sender, EventArgs e)
        {
            this.CastIdColumn.Renderer = new MappedImageRenderer(new Object[] { "check", GridResources.tick16, "newid", GridResources.star16 });
            this.CastIdColumn.AspectGetter = (row =>
                    {
                        CastEntry entry;

                        entry = (CastEntry)row;
                        if (String.IsNullOrEmpty(entry.CastId) == false)
                        {
                            if (entry.CastId .StartsWith(NewId ))
                            {
                                return ("newid");
                            }
                            return ("check");
                        }
                        else
                        {
                            return (null);
                        }
                    }
               );
            this.CastListView.CanExpandGetter = (castEntry =>
                    {
                        Episode episode;

                        episode = castEntry as Episode;
                        if (episode != null)
                        {
                            return ((episode.CastList != null) && (episode.CastList.Count > 0));
                        }
                        return (false);
                    }
                );
            this.CastListView.ChildrenGetter = (castEntry =>
                    {
                        return (((Episode)castEntry).CastList);
                    }
                );

            if (this.ShowAddEpisodeButton == false)
            {
                this.AddEpisodeButton.Enabled = false;
            }
        }

        private void AddCastButton_Click(object sender, EventArgs e)
        {
            CastMember castMember;

            castMember = new CastMember();
            castMember.Upc = this.Upc;
            castMember.DatabaseCastId = -1;
            using (EditCastForm editCastForm = new EditCastForm(castMember))
            {
                editCastForm.ShowDialog();
                if (editCastForm.DialogResult == DialogResult.OK)
                {
                    if (this.CastEntryAdded != null)
                    {
                        AddedEventArgs eventArgs;

                        eventArgs = new AddedEventArgs(castMember);
                        this.CastEntryAdded(this, eventArgs);
                    }
                }
            }
        }

        private void AddEpisodeButton_Click(object sender, EventArgs e)
        {
            Episode episode;

            episode = new Episode();
            episode.Upc = this.Upc;
            using (EditEpisodeForm form = new EditEpisodeForm(episode))
            {
                form.ShowDialog();
                if (form.DialogResult == DialogResult.OK)
                {
                    if (this.CastEntryAdded != null)
                    {
                        AddedEventArgs eventArgs;

                        eventArgs = new AddedEventArgs(episode);
                        this.CastEntryAdded(this, eventArgs);
                    }
                }
            }
        }

        private void EditRowButton_Click(object sender, EventArgs e)
        {
            if (this.CastListView.SelectedIndex != -1)
            {
                OLVListItem row;
                CastMember castMember;

                row = (OLVListItem)(this.CastListView.Items[this.CastListView.SelectedIndex]);
                castMember = row.RowObject as CastMember;
                if (castMember != null)
                {
                    using (EditCastForm form = new EditCastForm(castMember))
                    {
                        form.ShowDialog();
                        if (form.DialogResult == DialogResult.OK)
                        {
                            this.CastListView.RebuildAll(true);
                        }
                    }
                }
                else
                {
                    Episode episode;

                    episode = row.RowObject as Episode;
                    if (episode != null)
                    {
                        using (EditEpisodeForm form = new EditEpisodeForm(episode))
                        {
                            form.ShowDialog();
                            if (form.DialogResult == DialogResult.OK)
                            {
                                this.CastListView.RebuildAll(true);
                            }
                        }
                    }
                }
            }
            else
            {
                MessageBox.Show("Please select a row.");
            }
        }
    }
}
