﻿using System;
using System.Windows.Forms;

namespace DoenaSoft.DVDProfiler.CastDemoClient_V2
{
    internal partial class EditCastForm : Form
    {
        private CastMember m_CastMember;

        private String m_PreviousCastId;

        public EditCastForm(CastMember castMember)
        {
            m_CastMember = castMember;
            m_PreviousCastId = m_CastMember.CastId;
            InitializeComponent();
            this.FillName();
            this.RoleTextBox.Text = m_CastMember.Role;
            this.VoiceCheckBox.Checked = m_CastMember.Voice;
            this.CreditedAsTextBox.Text = m_CastMember.CreditedAs;
            this.SetCastIdTextBox();
            CastIdTextBox_TextChanged(this, EventArgs.Empty);
        }

        private void SetCastIdTextBox()
        {
            if ((String.IsNullOrEmpty(m_CastMember.CastId)==false) &&(m_CastMember.CastId.StartsWith(CastListControl.NewId)))
            {
                this.CastIdTextBox.Text = m_CastMember.CastId.Substring(CastListControl.NewId.Length);
            }
            else
            {
                this.CastIdTextBox.Text = m_CastMember.CastId;
            }
        }

        private void FillName()
        {
            this.FirstNameTextBox.Text = m_CastMember.FirstName;
            this.MiddleNameTextBox.Text = m_CastMember.MiddleName;
            this.LastNameTextBox.Text = m_CastMember.LastName;
        }

        private void AssignCastIdButton_Click(object sender, EventArgs e)
        {
            CastMember tempCastMember;

            if (this.CheckFirstName() == false)
            {
                return;
            }
            tempCastMember = new CastMember();
            tempCastMember.FirstName = this.FirstNameTextBox.Text;
            tempCastMember.MiddleName = this.MiddleNameTextBox.Text;
            tempCastMember.LastName = this.LastNameTextBox.Text;
            tempCastMember.CastId = m_CastMember.CastId;
            tempCastMember.Upc = m_CastMember.Upc;
            using (SearchForCastForm form = new SearchForCastForm(tempCastMember))
            {
                form.ShowDialog();
                if (form.DialogResult == DialogResult.OK)
                {
                    m_CastMember.FirstName = tempCastMember.FirstName;
                    m_CastMember.MiddleName = tempCastMember.MiddleName;
                    m_CastMember.LastName = tempCastMember.LastName;
                    m_CastMember.CastId = tempCastMember.CastId;
                    this.SetCastIdTextBox();
                    if (String.IsNullOrEmpty(this.CreditedAsTextBox.Text))
                    {
                        this.CreditedAsTextBox.Text = this.FirstNameTextBox.Text;
                        if (String.IsNullOrEmpty(this.MiddleNameTextBox.Text) == false)
                        {
                            this.CreditedAsTextBox.Text += " " + this.MiddleNameTextBox.Text;
                        }
                        if (String.IsNullOrEmpty(this.LastNameTextBox.Text) == false)
                        {
                            this.CreditedAsTextBox.Text += " " + this.LastNameTextBox.Text;
                        }
                    }
                    this.FillName();
                }
            }
        }

        private void AbortButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void OKButton_Click(object sender, EventArgs e)
        {
            if (this.CheckFirstName() == false)
            {
                return;
            }
            m_CastMember.FirstName = this.FirstNameTextBox.Text;
            m_CastMember.MiddleName = this.MiddleNameTextBox.Text;
            m_CastMember.LastName = this.LastNameTextBox.Text;
            m_CastMember.Role = this.RoleTextBox.Text;
            m_CastMember.Voice = this.VoiceCheckBox.Checked;
            m_CastMember.CreditedAs = this.CreditedAsTextBox.Text;
            //m_CastMember.CastId = this.CastIdTextBox.Text;
            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        private void CastIdTextBox_TextChanged(object sender, EventArgs e)
        {
            Boolean switcher;

            switcher = String.IsNullOrEmpty(this.CastIdTextBox.Text);
            this.CreditedAsTextBox.ReadOnly = switcher;
            this.FirstNameTextBox.ReadOnly = (switcher == false);
            this.MiddleNameTextBox.ReadOnly = (switcher == false);
            this.LastNameTextBox.ReadOnly = (switcher == false);
        }

        private void EditCastForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this.DialogResult == DialogResult.Cancel)
            {
                m_CastMember.CastId = m_PreviousCastId;
            }
        }

        private Boolean CheckFirstName()
        {
            if (String.IsNullOrEmpty(this.FirstNameTextBox.Text))
            {
                MessageBox.Show("Please enter at least a First Name!");
                return (false);
            }
            return (true);
        }
    }
}
