﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.OleDb;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace DoenaSoft.DVDProfiler.CastDemoClient_V2
{
    public partial class MainForm : Form
    {
        private const String NULL = "NULL";

        internal static Settings Settings;

        private List<CastEntry> m_RootElements;

        public MainForm()
        {
            InitializeComponent();
            this.CastListControl.ShowAddEpisodeButton = true;
        }

        private void MainForm_Load(object sender, System.EventArgs e)
        {
            GetSettings();
            GetData();
            this.CastListControl.Upc = this.UpcTextBox.Text;
            this.CastListControl.CastListView.Roots = m_RootElements;
            this.CastListControl.CastListView.RebuildAll(false);
            this.CastListControl.CastEntryAdded += this.CastListControl_CastEntryAdded;
        }

        private void GetData()
        {
            m_RootElements = new List<CastEntry>();
            Program.LocalConnection.Open();
            using (OleDbCommand command = Program.LocalConnection.CreateCommand())
            {
                command.CommandText = "SELECT * FROM vCast";
                using (OleDbDataReader castReader = command.ExecuteReader())
                {
                    if (castReader.HasRows)
                    {
                        Episode episode;
                        Int32 previousEpisodeId;

                        previousEpisodeId = -1;
                        episode = null;
                        while (castReader.Read())
                        {
                            CastMember castMember;
                            String temp;

                            this.UpcTextBox.Text = castReader.GetString(0);
                            this.TitleTextBox.Text = castReader.GetString(1);
                            if (castReader.IsDBNull(11))
                            {
                                //cast member is not assigned to an episode
                                episode = null;
                            }
                            else
                            {
                                Int32 newEpisodeId;

                                newEpisodeId = castReader.GetInt32(11);
                                if (previousEpisodeId != newEpisodeId)
                                {
                                    //next episode
                                    episode = null;
                                }
                                if (episode == null)
                                {
                                    previousEpisodeId = newEpisodeId;
                                    episode = new Episode();
                                    episode.Upc = this.UpcTextBox.Text;
                                    episode.EpisodeNumber = castReader.GetString(2);
                                    episode.EpisodeName = castReader.GetString(3);
                                    m_RootElements.Add(episode);
                                }
                            }
                            castMember = new CastMember();
                            castMember.Upc = this.UpcTextBox.Text;
                            castMember.FirstName = castReader.GetString(4);
                            if (castReader.IsDBNull(5) == false)
                            {
                                temp = castReader.GetString(5);
                                ClearNull(ref temp);
                                castMember.MiddleName = temp;
                            }
                            if (castReader.IsDBNull(6) == false)
                            {
                                temp = castReader.GetString(6);
                                ClearNull(ref temp);
                                castMember.LastName = temp;
                            }
                            if (castReader.IsDBNull(7) == false)
                            {
                                temp = castReader.GetString(7);
                                ClearNull(ref temp);
                                castMember.Role = temp;
                            }
                            castMember.Voice = castReader.GetBoolean(8);
                            if (castReader.IsDBNull(9) == false)
                            {
                                temp = castReader.GetString(9);
                                ClearNull(ref temp);
                                castMember.CreditedAs = temp;
                            }
                            if (castReader.IsDBNull(10) == false)
                            {
                                castMember.CastId = castReader.GetString(10);
                            }
                            castMember.DatabaseCastId = castReader.GetInt32(12);
                            if (episode != null)
                            {
                                if (episode.CastList == null)
                                {
                                    episode.CastList = new List<CastMember>();
                                }
                                episode.CastList.Add(castMember);
                            }
                            else
                            {
                                m_RootElements.Add(castMember);
                            }
                        }
                    }
                }
            }
            Program.LocalConnection.Close();
        }

        internal static void ClearNull(ref String field)
        {
            if (field == "Null")
            {
                field = "";
            }
        }

        void CastListControl_CastEntryAdded(object sender, AddedEventArgs e)
        {
            m_RootElements.Add(e.NewCastEntry);
            this.CastListControl.CastListView.Roots = m_RootElements;
            this.CastListControl.CastListView.RebuildAll(true);
        }

        private void GetSettings()
        {
            Settings = null;
            if (File.Exists("settings.xml"))
            {
                Settings = Settings.Deserialize("settings.xml");
            }
            if (Settings == null)
            {
                Settings = new Settings();
                using (LoginForm loginForm = new LoginForm(Settings))
                {
                    loginForm.ShowDialog();
                    if (loginForm.DialogResult == DialogResult.Cancel)
                    {
                        this.Close();
                    }
                }
                Settings.Serialize("settings.xml");
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Program.LocalConnection.Open();
            using (OleDbCommand command = Program.LocalConnection.CreateCommand())
            {
                Int32 orderNumber;

                command.Transaction = Program.LocalConnection.BeginTransaction();
                command.CommandText = "DELETE FROM tDVDxCast WHERE DVDId = '" + this.UpcTextBox.Text +  "'";
                command.ExecuteNonQuery();
                command.CommandText = "DELETE FROM tDivider WHERE DVDId = '" + this.UpcTextBox.Text + "'";
                command.ExecuteNonQuery();
                orderNumber = 0;
                foreach (CastEntry entry in m_RootElements)
                {
                    Episode episode;

                    episode = entry as Episode;
                    if (episode != null)
                    {
                        AddDividerToDatabase(episode, command, ref orderNumber);
                    }
                    else
                    {                        
                        AddCastMemberToDatabase(command, -1, ref orderNumber, (CastMember)entry);
                    }
                }
                command.Transaction.Commit();                
            }
            Program.LocalConnection.Close();
        }

        private void AddDividerToDatabase(Episode episode, OleDbCommand commmand, ref Int32 orderNumber)
        {
            Int32 dividerId;

            StringBuilder commandText;

            commandText = new StringBuilder("INSERT INTO tDivider (DVDId, [Number], Title) VALUES (");
            commandText.Append(PrepareTextForDb(this.UpcTextBox.Text));
            commandText.Append(", ");
            commandText.Append(PrepareTextForDb(episode.EpisodeNumber));
            commandText.Append(", ");
            commandText.Append(PrepareTextForDb(episode.EpisodeName));
            commandText.Append(")");
            commmand.CommandText = commandText.ToString();
            commmand.ExecuteNonQuery();
            commandText = new StringBuilder("SELECT @@identity FROM tDivider");
            commmand.CommandText = commandText.ToString();
            using (OleDbDataReader episodeReader = commmand.ExecuteReader(CommandBehavior.SingleResult))
            {
                episodeReader.Read();
                dividerId = episodeReader.GetInt32(0);
            }
            foreach (CastMember castMember in episode.CastList)
            {
                AddCastMemberToDatabase(commmand, dividerId, ref orderNumber, castMember);
            }
        }

        private void AddCastMemberToDatabase(OleDbCommand command, Int32 dividerId, ref  Int32 orderNumber, CastMember castMember)
        {
            StringBuilder commandText;

            if (castMember.DatabaseCastId == -1)
            {
                commandText = new StringBuilder("INSERT INTO tCast (LastName, MiddleName, FirstName) VALUES (");
                commandText.Append(PrepareOptionalTextForDb(castMember.LastName));
                commandText.Append(", ");
                commandText.Append(PrepareOptionalTextForDb(castMember.MiddleName));
                commandText.Append(", ");
                commandText.Append(PrepareTextForDb(castMember.FirstName));
                commandText.Append(")");
                command.CommandText = commandText.ToString();
                command.ExecuteNonQuery();
                commandText = new StringBuilder("SELECT @@identity FROM tDivider");
                command.CommandText = commandText.ToString();
                using (OleDbDataReader reader = command.ExecuteReader(CommandBehavior.SingleResult))
                {
                    reader.Read();
                    castMember.DatabaseCastId = reader.GetInt32(0);
                }
            }
            commandText = new StringBuilder("UPDATE tCast SET CastId = ");
            commandText.Append(PrepareOptionalTextForDb(castMember.CastId));
            commandText.Append(" WHERE Id = ");
            commandText.Append(castMember.DatabaseCastId);
            command.CommandText = commandText.ToString();
            command.ExecuteNonQuery();
            orderNumber++;
            commandText = new StringBuilder("INSERT INTO tDVDxCast (DVDId, CastId, DividerId, Role, CreditedAs, Voice, OrderNumber) VALUES (");
            commandText.Append(PrepareTextForDb(this.UpcTextBox.Text));
            commandText.Append(", ");
            commandText.Append(castMember.DatabaseCastId);
            commandText.Append(", ");
            if (dividerId == -1)
            {
                commandText.Append(NULL);
            }
            else
            {
                commandText.Append(dividerId);
            }
            commandText.Append(", ");
            commandText.Append(PrepareOptionalTextForDb(castMember.Role));
            commandText.Append(", ");
            commandText.Append(PrepareOptionalTextForDb(castMember.CreditedAs));
            commandText.Append(", ");
            commandText.Append(castMember.Voice.ToString());
            commandText.Append(", ");
            commandText.Append(orderNumber);
            commandText.Append(")");
            command.CommandText = commandText.ToString();
            command.ExecuteNonQuery();
        }

        internal static String PrepareTextForDb(String text)
        {
            return ("'" + ReplaceApostrophes(text) + "'");
        }

        internal static String ReplaceApostrophes(String text)
        {
            return (text.Replace("'", "''"));
        }

        internal static String PrepareOptionalTextForDb(String text)
        {
            if (String.IsNullOrEmpty(text))
            {
                return (NULL);
            }
            else
            {
                return (PrepareTextForDb(text));
            }
        }

        private static String PrepareCastIdForDb(String text)
        {
            if (String.IsNullOrEmpty(text))
            {
                return (NULL);
            }
            else
            {
                return (text);
            }
        }

        private void ContributeButton_Click(object sender, EventArgs e)
        {
            Program.OnlineConnection.Open();
            using (OleDbCommand command = Program.OnlineConnection.CreateCommand())
            {
                StringBuilder commandText;

                command.Transaction = Program.OnlineConnection.BeginTransaction();
                commandText = new StringBuilder("DELETE FROM tDVDxCast WHERE (DVDId =");
                commandText.Append(PrepareTextForDb(this.UpcTextBox.Text));
                commandText.Append(")");
                command.CommandText = commandText.ToString();
                command.ExecuteNonQuery();
                foreach (CastEntry castEntry in m_RootElements)
                {
                    Episode episode;

                    episode = castEntry as Episode;
                    if (episode != null)
                    {
                        if (episode.CastList != null)
                        {
                            foreach (CastMember castMember in episode.CastList)
                            {
                                ContributeCastMember(command, castMember);
                            }
                        }
                    }
                    else
                    {
                        ContributeCastMember(command, (CastMember)castEntry);
                    }
                }
                command.Transaction.Commit();
            }
            Program.OnlineConnection.Close();
            this.CastListControl.CastListView.Roots = m_RootElements;
            this.CastListControl.CastListView.RebuildAll(true);
        }

        private void ContributeCastMember(OleDbCommand command, CastMember castMember)
        {
            StringBuilder commandText;

            if (String.IsNullOrEmpty(castMember.CastId))
            {
                commandText = new StringBuilder("INSERT INTO tDVDxCast (DVDId, LastName, MiddleName, FirstName, Role, Voice) VALUES (");
                commandText.Append(PrepareTextForDb(this.UpcTextBox.Text));
                commandText.Append(", ");
                commandText.Append(PrepareOptionalTextForDb(castMember.LastName));
                commandText.Append(", ");
                commandText.Append(PrepareOptionalTextForDb(castMember.MiddleName));
                commandText.Append(", ");
                commandText.Append(PrepareTextForDb(castMember.FirstName));
                commandText.Append(", ");
                commandText.Append(PrepareOptionalTextForDb(castMember.Role));
                commandText.Append(", ");
                commandText.Append(castMember.Voice);
                commandText.Append(")");
                command.CommandText = commandText.ToString();
                command.ExecuteNonQuery();
            }
            else
            {
                if (castMember.CastId.StartsWith(CastListControl.NewId))
                {
                    String newId;

                    newId = castMember.CastId.Substring(CastListControl.NewId.Length);
                    commandText = new StringBuilder("INSERT INTO tCast SELECT Id, LastName, MiddleName, FirstName FROM tPendingContributions WHERE Id = ");
                    commandText.Append(newId);
                    command.CommandText = commandText.ToString();
                    command.ExecuteNonQuery();
                    commandText = new StringBuilder("DELETE FROM tPendingContributions WHERE Id = ");
                    commandText.Append(newId);
                    command.CommandText = commandText.ToString();
                    command.ExecuteNonQuery();
                    castMember.CastId = newId.ToString();
                }
                commandText = new StringBuilder("INSERT INTO tDVDxCast (DVDId, CastId, Role, CreditedAs, Voice) VALUES (");
                commandText.Append(PrepareTextForDb(this.UpcTextBox.Text));
                commandText.Append(", ");
                commandText.Append(castMember.CastId);
                commandText.Append(", ");
                commandText.Append(PrepareOptionalTextForDb(castMember.Role));
                commandText.Append(", ");
                commandText.Append(PrepareOptionalTextForDb(castMember.CreditedAs));
                commandText.Append(", ");
                commandText.Append(castMember.Voice);
                commandText.Append(")");
                command.CommandText = commandText.ToString();
                command.ExecuteNonQuery();
            }
        }

    }
}
